(PackageInfo named: 'Relationships') postscriptOfRemoval: '"below, add code to clean up after the unloading of this package"
Class removeInstVarName: ''relationships''.'!
(PackageInfo named: 'Relationships') preamble: '"below, add code to be run before the loading of this package"
| relationship |
relationship := ''relationships''.
(Class instVarNames contains: [:each | (each = relationship)] )
	ifFalse: [Class addInstVarName: relationship]. '!
SystemOrganization addCategory: #'Relationships-Tests'!
SystemOrganization addCategory: #Relationships!

TestCase subclass: #RelationshipTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Relationships-Tests'!

!RelationshipTest methodsFor: 'as yet unclassified' stamp: 'cn 12/8/2010 23:11'!
setUp
	| rm result |
	rm := RelationshipManager new.
	result := rm one: Parent toMany: Child relationshipName:'children' biDirectional: true.
	self should: ((Parent relationships at: 'children') = result).! !

!RelationshipTest methodsFor: 'as yet unclassified' stamp: 'cn 12/9/2010 00:10'!
tearDown
	| relationship rm |
	rm := RelationshipManager new.
	relationship := Parent relationships at: 'children'.
	rm removeRelationship: relationship.
	self should: ((Parent relationships at: 'children' ifAbsent: []) = nil).
	self should: ((Child relationships at: 'children' ifAbsent: []) = nil).
	! !

!RelationshipTest methodsFor: 'as yet unclassified' stamp: 'cn 12/9/2010 00:11'!
testGeneratedClasses
	self should: (Parent relationships includesKey: 'children').
	self should: (Parent methodDict includesKey: ('addToChildList:' asSymbol) ).
	self should: (Child relationships includesKey: 'children').
	self should: (Child methodDict includesKey: ('parent:' asSymbol) ).! !

!RelationshipTest methodsFor: 'as yet unclassified' stamp: 'cn 12/9/2010 16:00'!
testGeneratedMethodsChildAccess
	"Test the methods that have been generated on the child side"
	| parent parent2 child |
	"Create child and parent"
	child := Child new.
	parent := Parent new.
	child parent: parent.
	self should: (child parent = parent).
	self should: ((parent childList at: 1) = child).
	"Remove parent"
	child parent: nil.
	self should: (parent childList size = 0).
	"Switch parent"
	parent2 := Parent new.
	child parent: parent.
	self should: (parent childList size = 1).
	self should: (parent2 childList size = 0).
	child parent: parent2.
	self should: (parent childList size = 0).
	self should: (parent2 childList size = 1).! !

!RelationshipTest methodsFor: 'as yet unclassified' stamp: 'cn 12/9/2010 13:41'!
testGeneratedMethodsParentAccess
	"Test the methods that have been generated on the parent side"
	| result parent child |
	"Create parent"
	parent := Parent new.
	self should: parent childList size = 0.
	"Add child to parent"
	parent addToChildList: Child new.
	self should: parent childList size = 1.
	child := parent childList at: 1.
	self should: (parent = child parent).
	"Getter returns copied list"
	result := parent childList.
	result remove: child.
	self should: result size = 0.
	self should: parent childList size = 1.
	"Remove child from parent"
	parent removeFromChildList: child.
	self should: parent childList size = 0.
	self should: (child parent = nil).! !

!Class methodsFor: '*relationships' stamp: 'cn 11/16/2010 18:23'!
addRelationship: aRelationship
	relationships ifNil: [relationships := Dictionary new].
	relationships at: aRelationship relationshipName put: aRelationship.! !

!Class methodsFor: '*relationships' stamp: 'cn 11/16/2010 18:30'!
relationships
	"Answer the value of relationships"

	^ relationships! !

!Class methodsFor: '*relationships' stamp: 'cn 12/8/2010 23:56'!
removeRelationship: aRelationship
	relationships ifNil: [relationships := Dictionary new].
	relationships removeKey: (aRelationship relationshipName).! !

Object subclass: #Child
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Relationships-Tests'!

Object subclass: #Parent
	instanceVariableNames: 'generatedChildList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Relationships-Tests'!

!Parent methodsFor: 'accessing' stamp: 'cn 12/9/2010 16:32'!
addToChildList: anObject
	generatedChildList ifNil: [generatedChildList := OrderedCollection new].
	generatedChildList add: anObject.
	anObject parent: self.! !

!Parent methodsFor: 'accessing' stamp: 'cn 12/9/2010 16:32'!
childList
	generatedChildList ifNil: [generatedChildList := OrderedCollection new].
	 ^generatedChildList copy.! !

!Parent methodsFor: 'accessing' stamp: 'cn 12/9/2010 16:32'!
removeFromChildList: anObject
	generatedChildList ifNotNil: [generatedChildList remove: anObject ifAbsent: [^false.] ].
	anObject parent = self ifTrue: [anObject parent: nil].
	^true.! !

Object subclass: #RelationshipDescription
	instanceVariableNames: 'relationshipName useNameForAccessors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Relationships'!

RelationshipDescription subclass: #OneToManyRelationship
	instanceVariableNames: 'oneClass manyClass isBidirectional'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Relationships'!

!OneToManyRelationship methodsFor: 'generating' stamp: 'cn 12/9/2010 17:00'!
activate
	self 
		generateInstVarOneClass; 
		generateMethodsOneClass.
	
	isBidirectional ifTrue: [
		self 
			generateInstVarManyClass;
			generateMethodsManyClass].! !

!OneToManyRelationship methodsFor: 'accessing' stamp: 'cn 12/8/2010 23:44'!
classes
	"Classes that reference this relationship"
	| classes |
	classes := OrderedCollection new.
	classes add: oneClass.
	isBidirectional ifTrue: [classes add: manyClass].
	^ classes.! !

!OneToManyRelationship methodsFor: 'generating' stamp: 'cn 12/8/2010 19:59'!
generateInstVarManyClass
	"Generates the instance variable."
	| instVarName |
	"Names"
	instVarName := self instVarNameManyClass .
	"Instance var"
	(manyClass instVarNames contains: [:each | (each = instVarName)] ) ifFalse: [manyClass addInstVarName: instVarName].! !

!OneToManyRelationship methodsFor: 'generating' stamp: 'cn 12/8/2010 20:00'!
generateInstVarOneClass
	"Generates the instance variable."
	| instVarName |
	"Names"
	instVarName := self instVarNameOneClass .
	"Instance var"
	(oneClass instVarNames contains: [:each | (each = instVarName)] ) ifFalse: [oneClass addInstVarName: instVarName].
	! !

!OneToManyRelationship methodsFor: 'generating' stamp: 'cn 12/9/2010 14:51'!
generateMethodsManyClass
	"Generates the methods. Needs to be called after the instance variable has been created."
	| instVarName methodString |
	"Names"
	instVarName := self instVarNameManyClass .
	"Getter"
	methodString := self getterNameManyClass, '
	 ^' , instVarName.
	manyClass compile: methodString classified: #accessing.
	"Setter - does this need an instanceof check?"
	methodString := self setterNameManyClass, ' anObject
	| oldObject |
	anObject = ', instVarName , ' ifTrue: [^ self].
	oldObject := ', instVarName , '.
	', instVarName , ' := anObject.
	oldObject ifNotNil: [oldObject ', self removeNameOneClass , ' self].
	anObject ifNotNil: [
		(anObject ', self getterNameOneClass , ' contains: [:item | item = self]) 
			ifFalse: [anObject ', self setterNameOneClass , ' self]
	]'.
	manyClass compile: methodString classified: #accessing.! !

!OneToManyRelationship methodsFor: 'generating' stamp: 'cn 12/9/2010 14:11'!
generateMethodsOneClass
	"Generates the methods. Needs to be called after the instance variable has been created."
	| instVarName methodString |
	"Names"
	instVarName := self instVarNameOneClass.
	"Getter"
	methodString := self getterNameOneClass , '
	', instVarName , ' ifNil: [' , instVarName , ' := OrderedCollection new].
	 ^' , instVarName, ' copy.'.
	oneClass compile: methodString classified: #accessing.
	"Setter - does this need an instanceof check?"
	methodString := self setterNameOneClass , ' anObject
	', instVarName , ' ifNil: [' , instVarName , ' := OrderedCollection new].
	', instVarName , ' add: anObject.
	anObject ', self setterNameManyClass , ' self.'.
	oneClass compile: methodString classified: #accessing.
	"Remove"
	methodString := self removeNameOneClass , ' anObject
	', instVarName , ' ifNotNil: [' , instVarName , ' remove: anObject ifAbsent: [^false.] ].
	anObject ', self getterNameManyClass , ' = self ifTrue: [anObject ', self setterNameManyClass , ' nil].
	^true.'.
	oneClass compile: methodString classified: #accessing.
	! !

!OneToManyRelationship methodsFor: 'names' stamp: 'cn 12/8/2010 23:00'!
getterNameManyClass
	^ self varNameManyClass.! !

!OneToManyRelationship methodsFor: 'names' stamp: 'cn 12/8/2010 22:51'!
getterNameOneClass
	^ self varNameOneClass.! !

!OneToManyRelationship methodsFor: 'names' stamp: 'cn 12/8/2010 23:27'!
instVarNameManyClass
	^ 'generated', (self varNameManyClass capitalized).! !

!OneToManyRelationship methodsFor: 'names' stamp: 'cn 12/8/2010 23:27'!
instVarNameOneClass
	^ 'generated', (self varNameOneClass capitalized).! !

!OneToManyRelationship methodsFor: 'accessing' stamp: 'cn 11/13/2010 15:53'!
isBidirectional
	"Answer the value of isBidirectional"

	^ isBidirectional! !

!OneToManyRelationship methodsFor: 'accessing' stamp: 'cn 11/13/2010 15:53'!
isBidirectional: anObject
	"Set the value of isBidirectional"

	isBidirectional := anObject! !

!OneToManyRelationship methodsFor: 'accessing' stamp: 'cn 11/13/2010 15:53'!
manyClass
	"Answer the value of manyClass"

	^ manyClass! !

!OneToManyRelationship methodsFor: 'accessing' stamp: 'cn 11/16/2010 18:20'!
manyClass: anObject
	"Set the value of manyClass"
	manyClass := anObject.	! !

!OneToManyRelationship methodsFor: 'accessing' stamp: 'cn 11/13/2010 15:53'!
oneClass
	"Answer the value of oneClass"

	^ oneClass! !

!OneToManyRelationship methodsFor: 'accessing' stamp: 'cn 11/16/2010 18:20'!
oneClass: anObject
	"Set the value of oneClass"
	oneClass :=  anObject.! !

!OneToManyRelationship methodsFor: 'removing' stamp: 'cn 12/9/2010 00:08'!
remove
	self removeMethodsOneClass removeInstVarOneClass.
	self removeMethodsManyClass removeInstVarManyClass.! !

!OneToManyRelationship methodsFor: 'removing' stamp: 'cn 12/8/2010 22:25'!
removeInstVarManyClass
	"Remove instance var"
	manyClass removeInstVarName: self instVarNameManyClass.! !

!OneToManyRelationship methodsFor: 'removing' stamp: 'cn 12/8/2010 22:24'!
removeInstVarOneClass
	"Remove instance var"
	oneClass removeInstVarName: self instVarNameOneClass.! !

!OneToManyRelationship methodsFor: 'removing' stamp: 'cn 12/9/2010 00:09'!
removeMethodsManyClass
	"Getter"
	manyClass removeSelector: (self getterNameManyClass asSymbol).
	"Setter"
	manyClass removeSelector: (self setterNameManyClass asSymbol).! !

!OneToManyRelationship methodsFor: 'removing' stamp: 'cn 12/8/2010 22:59'!
removeMethodsOneClass
	"Getter"
	oneClass removeSelector: self getterNameOneClass asSymbol.
	"Setter"
	oneClass removeSelector: self setterNameOneClass asSymbol.
	"Remove"
	oneClass removeSelector: self removeNameOneClass asSymbol.! !

!OneToManyRelationship methodsFor: 'names' stamp: 'cn 12/8/2010 23:29'!
removeNameOneClass
	^ 'removeFrom', (self varNameOneClass capitalized), ':'.! !

!OneToManyRelationship methodsFor: 'names' stamp: 'cn 12/8/2010 23:19'!
setterNameManyClass
	^ (self varNameManyClass, ':').! !

!OneToManyRelationship methodsFor: 'names' stamp: 'cn 12/8/2010 23:28'!
setterNameOneClass
	^ 'addTo', (self varNameOneClass capitalized), ':'.! !

!OneToManyRelationship methodsFor: 'names' stamp: 'cn 12/9/2010 13:23'!
varNameManyClass
	| oneName |
	useNameForAccessors
		ifTrue: [oneName := relationshipName withFirstCharacterDownshifted] 
		ifFalse: [oneName := ((oneClass  name subStrings: ' ') at: 1) withFirstCharacterDownshifted].
	^ oneName.! !

!OneToManyRelationship methodsFor: 'names' stamp: 'cn 12/9/2010 16:39'!
varNameOneClass
	| listName |
	useNameForAccessors
		ifTrue: [listName := relationshipName withFirstCharacterDownshifted] 
		ifFalse: [listName := ((manyClass name subStrings: ' ') at: 1) withFirstCharacterDownshifted, 's'].
	^ listName.! !

!RelationshipDescription methodsFor: 'accessing' stamp: 'cn 12/8/2010 22:16'!
activate
	"generate methods and instance variables"
	self subclassResponsibility.! !

!RelationshipDescription methodsFor: 'accessing' stamp: 'cn 12/8/2010 23:43'!
classes
	"Classes that reference this relationship"
	self subclassResponsibility.! !

!RelationshipDescription methodsFor: 'accessing' stamp: 'cn 11/13/2010 15:55'!
relationshipName
	"Answer the value of relationshipName"

	^ relationshipName! !

!RelationshipDescription methodsFor: 'accessing' stamp: 'cn 11/13/2010 15:55'!
relationshipName: anObject
	"Set the value of relationshipName"

	relationshipName := anObject! !

!RelationshipDescription methodsFor: 'accessing' stamp: 'cn 12/8/2010 22:16'!
remove
	"generate methods and instance variables"
	self subclassResponsibility.! !

!RelationshipDescription methodsFor: 'accessing' stamp: 'cn 11/16/2010 19:24'!
useNameForAccessors
	"Answer the value of useNameForAccessors"

	^ useNameForAccessors! !

!RelationshipDescription methodsFor: 'accessing' stamp: 'cn 11/16/2010 19:24'!
useNameForAccessors: anObject
	"Set the value of useNameForAccessors"

	useNameForAccessors := anObject! !

Object subclass: #RelationshipManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Relationships'!

!RelationshipManager methodsFor: 'as yet unclassified' stamp: 'cn 11/10/2010 19:06'!
one: oneClass toMany: manyClass
	"Adds a one to many relationship between the two classes"
	| listName methodString |
	listName := ((manyClass name subStrings: ' ') at: 1) , 'List'.
	(oneClass instVarNames contains: [:each | (each = listName)] ) ifFalse: [oneClass addInstVarName: listName].
	methodString := 'get' , listName , '
	', listName , ' ifNil: [' , listName , ' := OrderedCollection new].
	 ^' , listName.
	oneClass compile: methodString classified: #accessing.! !

!RelationshipManager methodsFor: 'as yet unclassified' stamp: 'cn 12/8/2010 23:41'!
one: oneClass toMany: manyClass relationshipName: aString biDirectional: aBoolean
	"Adds a one to many relationship between the two classes"
	| relationship |
	relationship := OneToManyRelationship new.
	relationship relationshipName: aString. 
	relationship oneClass: oneClass.
	relationship manyClass: manyClass.
	relationship isBidirectional: aBoolean.
	relationship useNameForAccessors: false.
	oneClass addRelationship: relationship.
	aBoolean ifTrue: [manyClass addRelationship: relationship].
	relationship activate.
	^ relationship.
	! !

!RelationshipManager methodsFor: 'as yet unclassified' stamp: 'cn 12/8/2010 23:46'!
removeRelationship: aRelationship
	"Removes the relationship from the classes"
	aRelationship remove.
	aRelationship classes do: [:class | class removeRelationship: aRelationship].
	! !
